#include "liste.h"
/* Implémentation par liste à sentinelle
La liste contient toujours deux maillons, un
à la tête et un à la queue, appelés les
sentinelles. Ces maillons sont invisibles du
point de vue de l'utilisateur. */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <stdbool.h>

typedef struct liste {
   struct maillon* tete;
   struct maillon* queue;
   int taille;
} liste_t;

typedef struct maillon {
   T val;
   struct maillon* suiv;
   struct maillon* prec;
   liste_t* mere;   
} maillon_t;

liste_t* liste_vide(){
   liste_t* l = malloc(sizeof(liste_t));
   l->tete = malloc(sizeof(maillon_t));
   l->queue = malloc(sizeof(maillon_t));
   l->tete->mere = l;
   l->queue->mere = l;

   // TETE <-> QUEUE
   l->tete->suiv = l->queue;
   l->tete->prec = NULL;
   l->queue->prec = l->tete;
   l->queue->suiv = NULL;

   l->taille = 0;
   return l;
}

int taille(liste_t* l){
   return l->taille;
}

bool sentinelle(maillon_t* m){
   return m->prec == NULL || m->suiv == NULL;
}

maillon_t* tete(liste_t* l){
   if (taille(l) == 0){
      return NULL;
   }
   return l->tete->suiv;
}

maillon_t* queue(liste_t* l){
   if (taille(l) == 0){
      return NULL;
   }
   return l->queue->prec;
}

maillon_t* suivant(maillon_t* m){
   assert(!sentinelle(m));
   if (sentinelle(m->suiv)){
      return NULL;
   }
   return (m->suiv);
}

maillon_t* precedent(maillon_t* m){
   assert(!sentinelle(m));
   if (sentinelle(m->prec)){
      return NULL;
   }
   return (m->prec);
}

T contenu(maillon_t* m){
   assert(!sentinelle(m));
   return m->val;
}

void inserer(maillon_t* m, T x){
   assert(m->suiv != NULL);
   maillon_t* s = m->suiv;
   maillon_t* new = malloc(sizeof(maillon_t));
   new->val = x;
   // avant: m <-> s
   // après: m <-> new <-> s

   // liens entre m et new
   m->suiv = new;
   new->prec = m;

   // liens entre new et s 
   new->suiv = s;
   s->prec = new;

   liste_t* l = m->mere;
   new->mere = l;
   l->taille++;
}


void ajout_tete(liste_t* l, T x){
   inserer(l->tete, x);
}

void ajout_queue(liste_t* l, T x){
   inserer(l->queue->prec, x);
}

void supprimer(maillon_t* m){
   assert(!sentinelle(m));
   maillon_t* p = m->prec;
   maillon_t* s = m->suiv;
   // avant: p <-> m <-> s
   // après: p <-> s     et m supprimé

   p->suiv = s;
   s->prec = p;

   liste_t* l = m->mere;
   l->taille--;
   free(m);
}

void free_liste(liste_t* l){
   while(taille(l)>0){
      supprimer(tete(l));
   }
   free(l->tete);
   free(l->queue);
   free(l);
}

/* Affiche l de la tête vers la queue. Par exemple:
   1 - 2 - 3 - 4 - 5 

   Affiche "(vide)" pour la liste vide.
*/
void print_liste(liste_t* l){
   maillon_t* m = tete(l);
   while (m != NULL){
      printf("%s", contenu(m));
      if (suivant(m) != NULL){
         printf(" - ");
      }
      m = suivant(m);
   }
   printf("\n");
}