#pragma once
/* Structure de données abstraite: Liste chaînée */

#define T char*

typedef struct liste liste_t;
typedef struct maillon maillon_t;

/* Crée une liste vide */
liste_t* liste_vide();

/* Détermine la taille de l */
int taille(liste_t* l);

/* Renvoie le maillon à l'avant de l.
   Précondition: la taille de l est strictement positive */
maillon_t* tete(liste_t* l);

/* Renvoie le maillon à la fin de l.
   Précondition: la taille de l est strictement positive */
maillon_t* queue(liste_t* l);

/* Renvoie le maillon suivant m dans la liste à laquelle il
   appartient. Renvoie NULL si m est le dernier maillon (i.e. la queue) */
maillon_t* suivant(maillon_t* m);

/* Renvoie le maillon précédant m dans la liste à laquelle il
   appartient. Renvoie NULL si m est le premier maillon (i.e. la tête) */
maillon_t* precedent(maillon_t* m);

/* Renvoie le contenu de m */
T contenu(maillon_t* m);

/* Crée un maillon contenant x et l'ajoute en tête de l */
void ajout_tete(liste_t* l, T x);

/* Crée un maillon contenant x et l'ajoute en queue de l */
void ajout_queue(liste_t* l, T x);


/* Supprime m de la liste à laquelle il appartient */
void supprimer(maillon_t*m);


/* Insère un maillon entre m et suivant(m) contenant x.
   Si m est la queue de sa liste, alors le nouveau 
   maillon est ajouté à la queue, après m */
void inserer(maillon_t* m, T x);

/* Libère toute la mémoire allouée à l */
void free_liste(liste_t* l);

/* Affiche l de la tête vers la queue. Par exemple:
   1 - 2 - 3 - 4 - 5 

   Affiche "(vide)" pour la liste vide.
*/
void print_liste(liste_t* l);