#include <string.h>
#include <assert.h>
#include <stdbool.h>
#include <stdio.h>

#include "liste.h"


/* tests pour les listes doublement chaînées */
void test(){
	liste_t* l = liste_vide();
	// ETAT de l: (vide)
	assert(taille(l) == 0);
	maillon_t* m = tete(l);
	assert(m == NULL);

	ajout_tete(l, "bla");
	// ETAT de l: bla
	m = tete(l);
	assert(strcmp(contenu(m), "bla") == 0);
	assert(suivant(m) == NULL && precedent(m) == NULL);

	ajout_tete(l, "truc");
	// ETAT de l: truc-bla
	m = tete(l);
	assert(strcmp(contenu(m), "truc") == 0);
	assert(suivant(m) != NULL);
	assert(strcmp(contenu(suivant(m)), "bla") == 0);

	ajout_tete(l, "toto");
	// ETAT de l: toto - truc - bla
	assert(taille(l) == 3);
	m = tete(l);
	assert(strcmp(contenu(m), "toto") == 0);
	assert(suivant(m) != NULL);
	assert(strcmp(contenu(suivant(tete(l))), "truc") == 0);
	assert(suivant(suivant(m)) != NULL);


	supprimer(suivant(tete(l)));
	// ETAT de l: toto - bla
	assert(taille(l) == 2);
	assert(strcmp(contenu(tete(l)), "toto") == 0);
	assert(strcmp(contenu(queue(l)), "bla") == 0);

	assert(suivant(tete(l)) == queue(l));
	assert(precedent(queue(l)) == tete(l));

	inserer(tete(l), "foo");
	// ETAT de l: toto - foo - bla
	assert(taille(l) == 3);
	assert(strcmp(contenu(suivant(tete(l))), "foo") ==0);


	inserer(queue(l), "bar");
	assert(taille(l) == 4);
	// ETAT de l: toto - foo - bla - bar
	assert(strcmp(contenu(queue(l)), "bar") == 0);

	// vérification des liaisons
	assert(suivant(suivant(suivant(tete(l)))) == queue(l));
	assert(precedent(precedent(precedent(queue(l)))) == tete(l));


	// tous les tests ont réussi
	free_liste(l);
	printf("Tous les tests ont réussi\n");
}

/* Renvoie true si x est dans l, false sinon */
bool recherche(liste_t* l, T x){
	// attention, ici le type T est celui des
	// strings, on ne peut pas tester l'égalité
	// avec ==, il faut utiliser strcmp
	maillon_t* m = tete(l);
	while (m != NULL){
		if (strcmp(contenu(m), x) == 0){
			return true;
		}
		m = suivant(m);
	}
	return false;
}

/* Renvoie true si l est triée dans l'ordre croissant,
   false sinon */
bool est_triee(liste_t* l){
	maillon_t* m = tete(l);
	while (m != NULL && suivant(m) != NULL){
		if (strcmp(contenu(m), contenu(suivant(m)))>0){
			return false;
		}
		m = suivant(m);
	}
	return true;
}

// trie l dans l'ordre croissant, renvoie une nouvelle liste
// contenant le résultat, et vide l
liste_t* tri_selection(liste_t* l){
	liste_t* res = liste_vide();
	while(taille(l) > 0){
		// extraire le maillon de contenu maximal
		maillon_t* m_0 = tete(l); // meilleur maillon jusque là
		for(maillon_t* m = tete(l); m != NULL; m = suivant(m)){
			if (strcmp(contenu(m), contenu(m_0)) > 0){
				// m contient une plus grand valeur que m_0
				m_0 = m;
			}
		}
		// mettre l'élément maximal en tête de la liste en construction
		ajout_tete(res, contenu(m_0));
		supprimer(m_0);
	}	
	return res;
}


// trie l dans l'ordre croissant, renvoie une nouvelle liste
// contenant le résultat, et vide l
liste_t* tri_rapide(liste_t* l){
	if (taille(l) == 0){
		return liste_vide();
	}

	if (taille(l) == 1){
		liste_t* res = liste_vide();
		ajout_tete(res, contenu(tete(l)));
		supprimer(tete(l));
		return res;
	}

	//partitionner l en deux listes l_inf et l_sup
	//selon la valeur de tete(l)
	liste_t* l_inf = liste_vide();
	liste_t* l_sup = liste_vide();
	char* pivot = contenu(tete(l));
	supprimer(tete(l));
	while(taille(l)>0){
		maillon_t* m = tete(l);
		if (strcmp(contenu(m), pivot)<0){
			ajout_tete(l_inf, contenu(m));
		} else {
			ajout_tete(l_sup, contenu(m));
		}
		supprimer(tete(l));
	}

	// trier récursivement les deux listes
	liste_t* l_inf_triee = tri_rapide(l_inf);
	liste_t* l_sup_triee = tri_rapide(l_sup);

	// reconstruction de la liste totale:
	// l_inf_triee + [pivot] + l_sup_triee
	liste_t* res = liste_vide();
	for(maillon_t* m = tete(l_inf_triee); m != NULL; m = suivant(m)){
		ajout_queue(res, contenu(m));
	}
	ajout_queue(res, pivot);
	for(maillon_t* m = tete(l_sup_triee); m != NULL; m = suivant(m)){
		ajout_queue(res, contenu(m));
	}

	// libération mémoire et retour
	free_liste(l_inf);
	free_liste(l_sup);
	free_liste(l_inf_triee);
	free_liste(l_sup_triee);
	return res;
}

/* tests pour les deux fonctions de tri proposées */
void test_tris(){
	liste_t* l = liste_vide();
	ajout_tete(l, "raton");
	ajout_tete(l, "cheval");
	ajout_tete(l, "chat");
	ajout_tete(l, "chien");
	ajout_tete(l, "rat");
	ajout_tete(l, "raton");

	liste_t* l_triee = tri_selection(l);
	print_liste(l_triee);
	assert(est_triee(l_triee));

	liste_t* l2 = liste_vide();
	ajout_tete(l2, "raton");
	ajout_tete(l2, "cheval");
	ajout_tete(l2, "chat");
	ajout_tete(l2, "chien");
	ajout_tete(l2, "rat");
	ajout_tete(l2, "raton");

	liste_t* l2_triee = tri_rapide(l2);
	print_liste(l2_triee);
	assert(est_triee(l2_triee));

	free_liste(l);
	free_liste(l_triee);
	free_liste(l2);
	free_liste(l2_triee);

	printf("Les deux tris fonctionnent correctement\n");
}


int main(){
	test();
	test_tris();

}